package com.lantone.security.web;

import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.DataAuthDTO;
import com.lantone.common.vo.ServiceDataAuthVO;
import com.lantone.security.facade.DataAuthHandleFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @ClassName: DataAuthController
 * @Description: 数据权限API
 * @Author songxl
 * @Date 2021/7/21
 * @Version 1.0
 */
@RestController
@Api(value = "数据权限API", tags = { "数据权限API" })
@RequestMapping("/data/auth")
public class DataAuthController {
    @Autowired
    private DataAuthHandleFacade dataAuthHandleFacade;

    @ApiOperation(value = "新增数据权限[by:songxl]",
            notes = "dataAuthVO:数据权限对象，必填<br>" +
                    "roles:角色列表，必填<br>" +
                    "type:操作码 1：新增；2：修改 ；3：删除")
    @PostMapping("/addAuth")
    @ApiIgnore
    @Transactional
    public CommonResult<Boolean> addAuth(@RequestBody ServiceDataAuthVO serviceDataAuthVO) {
        return CommonResult.success(dataAuthHandleFacade.CRUDOperation(serviceDataAuthVO));
    }

    @ApiOperation(value = "修改数据权限[by:songxl]",
            notes = "dataAuthVO:数据权限对象，必填<br>" +
                    "roles:角色列表，必填<br>" +
                    "type:操作码 1：新增；2：修改；3：删除")
    @PostMapping("/updateAuth")
    @ApiIgnore
    @Transactional
    public CommonResult<Boolean> updateAuth(@RequestBody ServiceDataAuthVO serviceDataAuthVO) {
        return CommonResult.success(dataAuthHandleFacade.CRUDOperation(serviceDataAuthVO));
    }

    @ApiOperation(value = "删除数据权限[by:songxl]",
            notes = "dataAuthVO:数据权限对象，必填<br>" +
                    "roles:角色列表，必填<br>" +
                    "type:操作码 1：新增；2：修改；3：删除")
    @PostMapping("/deleteAuth")
    @ApiIgnore
    @Transactional
    public CommonResult<Boolean> deleteAuth(@RequestBody ServiceDataAuthVO serviceDataAuthVO) {
        return CommonResult.success(dataAuthHandleFacade.CRUDOperation(serviceDataAuthVO));
    }

    @ApiOperation(value = "通过数据权限id和角色id获取数据权限详情[by:songxl]",
            notes = "dataAuthVO:数据权限对象，必填<br>" +
                    "roles:角色列表，必填<br>" +
                    "type:操作码 1：新增；2：修改；3：删除")
    @PostMapping("/getDataAuthById")
    @ApiIgnore
    public CommonResult<DataAuthDTO> getDataAuthById(@RequestBody ServiceDataAuthVO serviceDataAuthVO) {
        return CommonResult.success(dataAuthHandleFacade.getDataAuthById(serviceDataAuthVO));
    }
}
